--[[-------------------------------------------------------------------------
Based on the oUF-Core classes and layout functions by Trond A Ekseth

  Copyright (c) 2006-2008, Trond A Ekseth
  Copyright (c) 2006-2008, Cargor (EU-Nozdormu)
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of cargBags nor the names of its contributors may
        be used to endorse or promote products derived from this
        software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

local _G = getfenv(0)
local select = select
local type = type
local tostring = tostring

-- add-on object
local cargBags = CreateFrame"Button"
local RegisterEvent = cargBags.RegisterEvent
local metatable = {__index = cargBags}

local slotid, i, k, temp, button, bag, name, bagArg
local style, cache
local styles = {}
local furui = {}
local found, start, finish

local select = select
local type = type
local pairs = pairs

local objects = {}
local bank = {}

local notInited = true

local dummy = function() end

-- Events
local OnEvent = function(self, event, ...)
	self[event](self, event, ...)
end

local bagUpdaters = {}

function cargBags:RegisterStyle(name, func)
	if(type(name) ~= "string") then return error("Bad argument #1 to 'RegisterStyle' (string expected, got %s)", type(name)) end
	if(type(func) ~= "table" and type(getmetatable(func).__call) ~= "function") then return error("Bad argument #2 to 'RegisterStyle' (table expected, got %s)", type(func)) end
	if(styles[name]) then return error("Style [%s] already registered.", name) end
	if(not style) then style = name end

	styles[name] = func
end

function cargBags:SetActiveStyle(name)
	if(type(name) ~= "string") then return error("Bad argument #1 to 'SetActiveStyle' (string expected, got %s)", type(name)) end
	if(not styles[name]) then return error("Style [%s] does not exist.", name) end

	furui[style] = cache
	cache = furui[name] or {}

	style = name
end

function cargBags:Spawn(name, container, handler)
	if(not name) then return error("Unable to create frame. No name was defined.") end
	if(not style) then return error("Unable to create frame. No styles have been registered.") end

	local style = styles[style]
	local object = CreateFrame("Button", name, UIParent)

	object = setmetatable(object, metatable)

	object.Name = name
	if(handler and type(handler) == "string") then
		object.Handler = cargBags.Handler[handler] or cargBags.Handler['Standard']
	else
		object.Handler = handler or cargBags.Handler['Standard']
	end
	if(not object.Handler.name) then return error("Unable to create frame. No handler found.") end
	object.Bags = {}
	object.bagIDs = {}
	object.SlotWidth = 37
	object.SlotHeight = 37
	object.SlotSpacing = 1
	object.SlotAnchor = "TOPLEFT"
	object.Height = 0
	object.Width = 0

	style(object, type)

	if(object.Container) then object.Container.num = 0 end

	for _, bagArg in pairs(container) do
		found, _, start, finish = string.find(bagArg, "([0-9]+)-([0-9]+)")
		if(found) then
			for i = start, finish do
				cargBags:AddContainer(object, i)
			end
		else
			cargBags:AddContainer(object, bagArg)
		end
	end

	object:SetScript("OnEvent", OnEvent)
	
	object:RegisterEvent"PLAYER_ENTERING_WORLD"
	object:RegisterEvent"BAG_UPDATE"
	object:RegisterEvent"ITEM_LOCK_CHANGED"
	object:RegisterEvent"BAG_UPDATE_COOLDOWN"

	object.initHandler = true
	if(object.Handler['Init']) then object:Handler("Init", object) end
	tinsert(objects, object)
	object:Hide()

	return object
end

local function CheckBankClose()
	for object, _ in pairs(bank) do
		if(object:IsShown()) then return nil end
	end
	CloseBankFrame()
end

function cargBags:RegisterBankEvents()
	BankFrame:UnregisterEvent("BANKFRAME_OPENED")
	bank[self] = true
	self:RegisterEvent"BANKFRAME_OPENED"
	self:RegisterEvent"BANKFRAME_CLOSED"
	self:RegisterEvent"PLAYERBANKBAGSLOTS_CHANGED"
	self:SetScript("OnHide", CheckBankClose)
end

function cargBags:AddContainer(self, bagArg, start, finish)
	if(string.find(bagArg,"(.+):(.+):(.+)")) then
		_, _, bagID, start, finish = string.find(bagArg,"(.+):(.+):(.+)")
	elseif(string.find(bagArg,"(.+):(.+)")) then
		_, _, bagID, start = string.find(bagArg,"(.+):(.+)")
	elseif(tonumber(bagArg)) then
		bagID = bagArg
	end

	if(not bagID) then return error("AddContainer() - No Bag ID specified") end
	local bag = CreateFrame("Frame", nil, self.Container)
	bag.id = tonumber(bagID)
	bag:SetID(bag.id)
	bag.start = tonumber(start)
	bag.finish = tonumber(finish)
	if(bag.id == BANK_CONTAINER and not self.Bank) then
		self.Bank = true
		self:RegisterBankEvents()
	end
	tinsert(self.Bags, bag)
	self.bagIDs[bag.id] = bag
end

function cargBags:BAG_UPDATE(event, bagID, updateNeeded)
	if(bagID and not self.bagIDs[bagID]) then return nil end
	for name, func in pairs(bagUpdaters) do
		if(self[name] and func) then func(self, event, bagID, updateNeeded) end
	end
	self.initHandler = false
end
cargBags.PLAYER_ENTERING_WORLD = cargBags.BAG_UPDATE

function cargBags:UpdateHeight()
	self:SetHeight(self.Container:GetHeight() + self.Height)
	self:SetWidth(self.Container:GetWidth() + self.Width)
end

function cargBags:SetHandler(handler, ...)
	if(handler and type(handler) == "table") then
		self.Handler = handler
	else
		self.Handler = cargBags.Handler[handler] or cargBags.Handler['Standard']
	end
	if(not self.Handler.name) then return error("Unable to create frame. No handler found.") end
	self.initHandler = true
	self:Handler("Init", self, ...)
	self:BAG_UPDATE()
end

cargBags.version = "0.4"
cargBags.objects = objects
cargBags.bagUpdaters = bagUpdaters
cargBags.Handler = {}
_G.cargBags = cargBags